package openfire;

import rocks.xmpp.core.XmppException;
import rocks.xmpp.core.session.ConnectionConfiguration;
import rocks.xmpp.core.session.TcpConnectionConfiguration;
import rocks.xmpp.core.session.XmppClient;
import rocks.xmpp.core.session.XmppSessionConfiguration;
import rocks.xmpp.core.stanza.model.Presence;
import rocks.xmpp.extensions.muc.ChatRoom;
import rocks.xmpp.extensions.muc.ChatService;
import rocks.xmpp.extensions.muc.MultiUserChatManager;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class App {

    public static final String USER_NAME = "admin";
    public static final String USER_PASSWORD = "admin";
    public static final String USER_RESOURCE_PREFIX = "session-";
    public static final String USER_NICKNAME_PREFIX = "Nickname-";
    public static final Presence USER_PRESENCE = new Presence(Presence.Show.CHAT, (byte) 16);

    public static void main(String[] args) throws Exception {

        if (args.length != 6) {
            System.out.println("Invalid number of arguments.");
            System.out.println("The arguments are: <host> <port> <domain> <service-name> <room-name> <session-count>");
            return;
        }
        String host = args[0];
        int port = Integer.parseInt(args[1]);
        String domain = args[2];
        String serviceName = args[3];
        String roomName = args[4];
        int sessionCount = Integer.parseInt(args[5]);

        Random randomiser = new Random(System.currentTimeMillis());
        List<XmppClient> sessionList = new ArrayList<>(400);

        ConnectionConfiguration connectionConfiguration = TcpConnectionConfiguration.builder()
                .hostname(host)
                .port(port)
                .secure(false)
                .build();

        while (true) {
            for (int count = 0; count < sessionCount; ++count) {
                String resource = USER_RESOURCE_PREFIX + randomiser.nextInt(9999);
                System.out.println("Connecting with resource: " + resource);
                XmppClient session = connectXmppServer(connectionConfiguration, domain);
                session.login(USER_NAME, USER_PASSWORD, resource);

                ChatRoom room = enterRoom(session, serviceName, roomName, USER_NICKNAME_PREFIX + resource);
                room.sendMessage("Hello, Room!");
                sessionList.add(session);
            }

            pauseForDuration(4);
            System.out.println("Closing all connections...");
            for (XmppClient session : sessionList) {
                session.close();
            }
            sessionList.clear();
            System.out.println("Performing another cycle of connections................");
        }
    }

    private static XmppClient connectXmppServer(ConnectionConfiguration connectionConfiguration, String domain) throws XmppException {
        XmppSessionConfiguration sessionConfiguration = XmppSessionConfiguration.builder()
                .authenticationMechanisms(new String[]{"PLAIN"})
                .defaultResponseTimeout(Duration.ofSeconds(20))
                .initialPresence(() -> USER_PRESENCE)
                .build();
        XmppClient session = XmppClient.create(domain, sessionConfiguration, connectionConfiguration);
        session.connect();
        return session;
    }


    private static ChatRoom enterRoom(XmppClient session, String serviceName, String roomName, String nickname) throws XmppException {
        ChatService service = getMucService(session, serviceName);
        ChatRoom room = getMucRoom(service, roomName);
        room.enter(nickname);
        return room;
    }

    private static ChatService getMucService(XmppClient session, String serviceName) throws XmppException {
        MultiUserChatManager mucManager = session.getManager(MultiUserChatManager.class);
        for (ChatService service : mucManager.discoverChatServices().getResult()) {
            if (service.getAddress().getDomain().equals(serviceName)) {
                return service;
            }
        }
        throw new XmppException("MUC service not found");
    }

    private static ChatRoom getMucRoom(ChatService service, String roomName) throws XmppException {
        for (ChatRoom room : service.discoverRooms().getResult()) {
            if (room.getName().equals(roomName)) {
                return room;
            }
        }
        throw new XmppException("MUC room not found in service " + service);
    }

    private static void pauseForDuration(int delay) throws Exception {
        System.out.println("Waiting for " + delay + " seconds...");
        Thread.sleep(1000 * delay);
    }
}
